﻿using DG.Tweening;
using UnityEngine;
using UnityEngine.Audio;

[System.Serializable]
public class AudioEntry {
    public AudioClip clip;
    public Vector2 distortion;
    public float delay;
}

public class AudioController : MonoBehaviour {
    [SerializeField]
    private AudioMixer audioMixer;
    [SerializeField]
    private AudioSource voiceAudioSource;
    public AudioSource musicAudioSource;
    public AudioSource sfxAudioSource;

    public void TransitionTo(string snapshotName, float duration) {
        AudioMixerSnapshot snapshot = audioMixer.FindSnapshot(snapshotName);
        if(snapshot != null) {
            snapshot.TransitionTo(duration);
        }
    }

    public void PlayVoice(AudioEntry entry) {
        //audioMixer.SetFloat("VoiceDistortion", Random.Range(entry.distortion.x, entry.distortion.y));

        var newSource = GameObject.Instantiate(voiceAudioSource);
        newSource.clip = entry.clip;

        if (entry.delay == 0) newSource.Play();
        else newSource.PlayDelayed(entry.delay);

        var clipLength = entry.clip.length;
        var destroyDelay = 0.1f;

        DOVirtual.DelayedCall(entry.delay + clipLength + destroyDelay, () => { Destroy(newSource.gameObject); });
    }

    public void PlayBGM(AudioClip clip) {
        var newSource = GameObject.Instantiate(musicAudioSource);
        newSource.clip = clip;
        newSource.volume = musicAudioSource.volume;
        newSource.time = musicAudioSource.time;
        newSource.DOFade(0f, 1f);

        musicAudioSource.volume = 0f;
        musicAudioSource.time = 0;
        musicAudioSource.clip = clip;
        musicAudioSource.DOFade(1f, 1f);
    }

    public void PlaySFX(AudioClip clip, float delay = 0f, bool loop = false) {
        var newSource = GameObject.Instantiate(sfxAudioSource);
        newSource.clip = clip;

        if (delay == 0) newSource.Play();
        else newSource.PlayDelayed(delay);

        var clipLength = clip.length;
        newSource.loop = loop;

        var destroyDelay = 0.1f;

        DOVirtual.DelayedCall(delay + clipLength + destroyDelay, () => { Destroy(newSource.gameObject); });
    }
}
