﻿using System.Collections;
using UnityEngine;

public class WaveLine : MonoBehaviour {
    public Color c1 = Color.green;
    public int lengthOfLineRenderer = 20;
    [Range(-1,1)]
    public float yOffset = 0.01f;

    public Waves.WaveType Wave;

    private float waveScale = 0.3f;
    private Vector3 startOffset;
    private float waveLength;
    private float waveHeight;
    private LineRenderer lineRenderer;
    private float noiseMultiplier;

    public void SetProperties(float matchHeight, float matchPeriod) {
        waveHeight = matchHeight;
        waveLength = matchPeriod;
    }

    public void SetWaveScale(float s) {
        waveScale = 0.3f*s;
    }

    public void SetNoise(float alpha) {
        noiseMultiplier = alpha;
    }

    public void SetType(Waves.WaveType type) {
        Wave = type;
    }

    void Start() {
        lineRenderer = gameObject.AddComponent<LineRenderer>();
        lineRenderer.material = new Material(Shader.Find("GUI/Text Shader"));
        lineRenderer.positionCount = lengthOfLineRenderer;
        lineRenderer.startColor = lineRenderer.endColor = c1;
        lineRenderer.startWidth = lineRenderer.endWidth = 0.05f;
        startOffset = transform.position;
    }
    
    protected virtual void Update() {
        int i = 0;
        float time = Time.time;
        while (i < lengthOfLineRenderer) {
            float x = i * 0.1f;
            float random = Random.Range(-noiseMultiplier,noiseMultiplier);
            Vector3 pos = startOffset+new Vector3(x*0.3f, (yOffset+Wave.GetY(i, time, waveHeight, waveLength)+random)*waveScale, 0);
            lineRenderer.SetPosition(i, pos);
            i++;
        }
    }

    public void Toggle(bool toggle) {
        if (lineRenderer == null) return;
        lineRenderer.startColor = lineRenderer.endColor = toggle ? Color.white : c1;
    }

    void OnEnable() {
        if (lineRenderer == null) return;
        StartCoroutine(FadeIn());
    }

    private IEnumerator FadeIn() {
        float time = 0;
        var maxTime = 0.3f;
        while (time < maxTime) {
            var percentage = time / maxTime;

            var color = lineRenderer.startColor;
            color.a = percentage;

            lineRenderer.startColor = lineRenderer.endColor = color;
            time += Time.deltaTime;
            yield return null;
        }
    }
}
