﻿using UnityEngine;

namespace Waves {
    public enum WaveTypeLink {
        Sine = 0,
        CustomSine = 1,
        Perlin1 = 2,
        Perlin2 = 3,
        Perlin3 = 4
    }

    public abstract class WaveType {
        public abstract float GetY(int i, float time, float waveHeight, float waveLength);
        public abstract Vector2 MinMaxHeight();
        public abstract Vector2 MinMaxLength();
    }

    public class Sine : WaveType {
        public override float GetY(int i, float time, float waveHeight, float waveLength) {
            return Mathf.Sin((i * waveLength) + time) * waveHeight;
        }

        public override Vector2 MinMaxHeight() {
            return new Vector2(2f, 2.7f);
        }

        public override Vector2 MinMaxLength() {
            return new Vector2(0.1f, 0.25f);
        }
    }

    public class CustomSine : WaveType {
        public override float GetY(int i, float time, float waveHeight, float waveLength) {
            return Mathf.Sin((i * waveLength) + time) * waveHeight + Mathf.Cos(i * 0.15f * waveLength) * 0.5f;
        }

        public override Vector2 MinMaxHeight() {
            return new Vector2(2f, 2.7f);
        }

        public override Vector2 MinMaxLength() {
            return new Vector2(0.1f, 0.25f);
        }
    }

    public class Perlin1 : WaveType {
        public override float GetY(int i, float time, float waveHeight, float waveLength) {
            return (Mathf.PerlinNoise(0, i * waveLength) * 2.0f - 0.5f) * waveHeight;
        }

        public override Vector2 MinMaxHeight() {
            return new Vector2(2f, 3f);
        }

        public override Vector2 MinMaxLength() {
            return new Vector2(0.05f, 0.25f);
        }
    }

    public class Perlin2 : WaveType {
        public override float GetY(int i, float time, float waveHeight, float waveLength) {
            return (Mathf.Sin((i * waveLength) + time) + (Mathf.PerlinNoise(0, i * waveLength) * 3.0f) / 4f) * waveHeight;
        }

        public override Vector2 MinMaxHeight() {
            return new Vector2(2f, 2.7f);
        }

        public override Vector2 MinMaxLength() {
            return new Vector2(0.05f, 0.3f);
        }
    }

    public class Perlin3 : WaveType {
        public override float GetY(int i, float time, float waveHeight, float waveLength) {
            return Mathf.Cos(i * waveLength) * Mathf.PerlinNoise(0, i * waveLength) * waveHeight;
        }

        public override Vector2 MinMaxHeight() {
            return new Vector2(2.0f, 3.5f);
        }

        public override Vector2 MinMaxLength() {
            return new Vector2(0.05f, 0.5f);
        }
    }
}