﻿using System.Collections;
using UnityEngine;
using XInputDotNetPure;

public class DeathEvent : VignetteEvent {
    public AudioController audioController;
    public AudioClip gameMusic;
    public GameObject leftLight;
    public GameObject rightLight;
    public AudioClip powerOff;
    public AudioClip lostHimAudio;
    public GameObject ui;

    [SerializeField]
    private GameObject flareObject;

    private bool isVibrating = false;
    
    public override void Play() {
        StartCoroutine(IEDeath());
    }

    public void Vibrate(float duration) {
        StartCoroutine(IEVibrate(duration));
    }

    private IEnumerator IEVibrate(float duration) {
        isVibrating = true;
        yield return new WaitForSeconds(duration);
        isVibrating = false;
    }

    protected void FixedUpdate() {
        if (isVibrating) {
            GamePad.SetVibration(PlayerIndex.One, 0.6f, 0.5f);
        } else {
            GamePad.SetVibration(PlayerIndex.One, 0, 0);
        }
    }
    
    private IEnumerator FlareRoutine() {
        flareObject.SetActive(true);

        var time = 0f;
        var maxTime = 0.8f;

        while (time < maxTime) {
            time += Time.deltaTime;
            yield return null;
        }
        
        camPostProcess.profile.eyeAdaptation.enabled = false;
    }

    private IEnumerator IEDeath() {
        SetVignetteSettings(new Vector2(0.5f, 0.5f), 0.15f, Color.black);
        yield return new WaitForSeconds(3);
        audioController.PlaySFX(powerOff);

        Vibrate(1.2f);
        StartCoroutine(Flicker(ui, 5));

        yield return new WaitForSeconds(1.5f);

        StartCoroutine(Flicker(leftLight, 5));
        StartCoroutine(Flicker(rightLight, 5));

        yield return new WaitForSeconds(2);
        StartCoroutine(FlareRoutine());
        yield return new WaitForSeconds(2);
        
        audioController.musicAudioSource.clip = gameMusic;
        audioController.musicAudioSource.loop = false;
        audioController.musicAudioSource.Play();

        BreathingSystem breath = FindObjectOfType<BreathingSystem>();
        if (breath != null) {
            breath.CancelInvoke();
        }

        yield return new WaitForSeconds(3);

        audioController.PlaySFX(lostHimAudio);
        
        yield return new WaitForSeconds(7);

        OnComplete(this);
    }
}
