﻿using UnityEngine;
using UnityEngine.UI;
using DG.Tweening;
using System.Collections;

public class BloodPressureDisplay : MonoBehaviour {
    private float currentSys = 119;
    private float currentDia = 82;

    [SerializeField]
    private Text[] sysNumbers;
    [SerializeField]
    private Text[] diaNumbers;

    private Vector3[] orignalSysPositions;
    private Vector3[] orignalDiaPositions;

    private void Awake() {
        orignalSysPositions = new Vector3[sysNumbers.Length];
        for (int i = 0; i < orignalSysPositions.Length; i++) {
            orignalSysPositions[i] = sysNumbers[i].transform.localPosition;
        }

        orignalDiaPositions = new Vector3[diaNumbers.Length];
        for (int i = 0; i < orignalDiaPositions.Length; i++) {
            orignalDiaPositions[i] = diaNumbers[i].transform.localPosition;
        }

        SetDisplay(sysNumbers, orignalSysPositions, (int)currentSys);
        SetDisplay(diaNumbers, orignalDiaPositions, (int)currentDia);

        StartCoroutine(FluctuateSys());
        StartCoroutine(FluctuateDia());
    }

    private IEnumerator FluctuateSys() {
        while (true) {
            yield return new WaitForSeconds(Random.Range(8f, 20f));

            var value = 1;
            if (Random.value > 0.5f) value = -1;
            if (Random.value > 0.85f) value *= 2;

            currentSys += value;
            currentSys = Mathf.Clamp(currentSys, 100, 150);
            SetDisplay(sysNumbers, orignalSysPositions, (int)currentSys);
        }
    }

    private IEnumerator FluctuateDia() {
        while (true) {
            yield return new WaitForSeconds(Random.Range(8f, 20f));
            
            var value = 1;
            if (Random.value > 0.5f) value = -1;
            if (Random.value > 0.85f) value *= 2;

            currentDia += value;
            currentDia = Mathf.Clamp(currentDia, 76, 96);
            SetDisplay(diaNumbers, orignalDiaPositions, (int)currentDia);
        }
    }

    public void SetMax(float maxSys, float maxDia) {
        DOVirtual.Float(currentSys, maxSys, Random.Range(8f, 20f), (value) => {
            currentSys = value;
            SetDisplay(sysNumbers, orignalSysPositions, (int)currentSys);
        }).SetEase((Ease)Random.Range(0, 34));

        DOVirtual.Float(currentDia, maxDia, Random.Range(8f, 20f), (value) => {
            currentDia = value;
            SetDisplay(diaNumbers, orignalDiaPositions, (int)currentDia);
        }).SetEase((Ease)Random.Range(0, 34));
    }

    private void SetDisplay(Text[] textfields, Vector3[] startPositions, int num) {
        var value = num.ToString().ToCharArray();
        for (var i = 0; i < value.Length; i++) {
            var currentText = textfields[i].text;
            var newText = value[i].ToString();

            if (currentText != newText) {
                AnimateTextfield(textfields[i], startPositions[i], value[i]);
            }
        }
    }

    private void AnimateTextfield(Text textfield, Vector3 startPos, char value) {
        var textTransform = textfield.transform;

        var sequence = DOTween.Sequence();
        sequence.Insert(0f, textTransform.DOLocalMoveY(startPos.y - 20f, 0.25f));
        sequence.Insert(0f, textfield.DOFade(0f, 0.25f));
        sequence.OnComplete(() => {
            textfield.text = value.ToString();
            textTransform.localPosition = new Vector3(startPos.x, startPos.y + 20f, startPos.z);

            textTransform.DOLocalMoveY(startPos.y, 0.25f);
            textfield.DOFade(1f, 0.25f);
        });

        sequence.Play();
    }
}
